==> desiderata-1.0.0.jar: clj-kondo.exports/systems.thoughtfull/desiderata/systems/thoughtfull/desiderata.clj <==
(ns systems.thoughtfull.desiderata
  (:refer-clojure :exclude [defrecord]))

(defn- parse-opts
  [opts+specs]
  (loop [[k v & s :as specs] opts+specs
         defaults nil
         opts []]
    (if (and (seq specs) (keyword? k))
      (if (= (name k) "defaults")
        (recur s v opts)
        (recur s defaults (conj opts k v)))
      [defaults opts specs])))

(defn- parse-initializer
  [name specs]
  (loop [[s & ss :as specs] specs
         not-initializer []]
    (if (seq specs)
      (if (and (list? s) (= (first s) name))
        [s (into not-initializer ss)]
        (recur ss (conj not-initializer s)))
      [nil not-initializer])))

(defmacro defrecord
  [name docstring? & [fields & opts+specs]]
  (let [[docstring fields opts+specs] (if (string? docstring?)
                                        [docstring? fields opts+specs]
                                        [nil docstring? (cons fields opts+specs)])
        [defaults opts specs] (parse-opts opts+specs)
        [initializer specs] (parse-initializer name specs)
        [init-params & init-body] (or (next initializer) `[[this#] this#])
        pfactory (symbol (format "->%s" name))
        factory (symbol (format "map->%s" name))]
    `(let [defaults# (fn [] ~defaults)
           record# (clojure.core/defrecord ~name ~fields ~@opts ~@specs)
           pfactory# ~pfactory
           factory# ~factory
           initializer# (fn ~init-params
                          (let [this# (do ~@init-body)]
                            (when-not (instance? record# this#)
                              (throw (java.lang.IllegalStateException.
                                       (str "Initializer should return instance of " '~name))))
                            this#))]
       ^{:clj-kondo/ignore [:redefined-var]}
       (defn ~pfactory
         {:doc (str (:doc (meta (var ~pfactory)))
                 (when-let [docstring# ~docstring]
                   (format "\n\n  %s" docstring#)))}
         [~@fields]
         (let [defaults# (defaults#)
               extra-keys# (into #{} (remove ~(into #{} (map keyword) fields)) (keys defaults#))]
           (initializer# (merge (pfactory# ~@fields) (select-keys defaults# extra-keys#)))))
       ^{:clj-kondo/ignore [:redefined-var]}
       (defn ~factory
         {:doc (str (:doc (meta (var ~factory)))
                 (when-let [docstring# ~docstring]
                   (format "\n\n  %s" docstring#)))
          :arglists '([& {:keys [~@fields]}])}
         [& {:as args#}]
         (initializer# (factory# (merge (defaults#) args#))))
       record#)))

==> desiderata-1.1.0.jar: clj-kondo.exports/systems.thoughtfull/desiderata/systems/thoughtfull/desiderata.clj <==
(ns systems.thoughtfull.desiderata
  (:refer-clojure :exclude [defrecord]))

(defn- parse-opts
  [opts+specs]
  (loop [[k v & s :as specs] opts+specs
         defaults nil
         opts []]
    (if (and (seq specs) (keyword? k))
      (if (= (name k) "defaults")
        (recur s v opts)
        (recur s defaults (conj opts k v)))
      [defaults opts specs])))

(defn- parse-initializer
  [name specs]
  (loop [[s & ss :as specs] specs
         not-initializer []]
    (if (seq specs)
      (if (and (list? s) (= (first s) name))
        [s (into not-initializer ss)]
        (recur ss (conj not-initializer s)))
      [nil not-initializer])))

(defmacro defrecord
  [name docstring? & [fields & opts+specs]]
  (let [[docstring fields opts+specs] (if (string? docstring?)
                                        [docstring? fields opts+specs]
                                        [nil docstring? (cons fields opts+specs)])
        [defaults opts specs] (parse-opts opts+specs)
        [initializer specs] (parse-initializer name specs)
        [init-params & init-body] (or (next initializer) `[[this#] this#])
        pfactory (symbol (format "->%s" name))
        factory (symbol (format "map->%s" name))]
    `(let [defaults# (fn [] ~defaults)
           record# (clojure.core/defrecord ~name ~fields ~@opts ~@specs)
           pfactory# ~pfactory
           factory# ~factory
           initializer# (fn [this#]
                          (let [{:keys [~@fields]} this#
                                ~init-params [this#]
                                this# (do ~@init-body)]
                            (when-not (instance? record# this#)
                              (throw (java.lang.IllegalStateException.
                                       (str "Initializer should return instance of " '~name))))
                            this#))]
       ^{:clj-kondo/ignore [:redefined-var]}
       (defn ~pfactory
         {:doc (str (:doc (meta (var ~pfactory)))
                 (when-let [docstring# ~docstring]
                   (format "\n\n  %s" docstring#)))}
         [~@fields]
         (let [defaults# (defaults#)
               extra-keys# (into #{} (remove ~(into #{} (map keyword) fields)) (keys defaults#))]
           (initializer# (merge (pfactory# ~@fields) (select-keys defaults# extra-keys#)))))
       ^{:clj-kondo/ignore [:redefined-var]}
       (defn ~factory
         {:doc (str (:doc (meta (var ~factory)))
                 (when-let [docstring# ~docstring]
                   (format "\n\n  %s" docstring#)))
          :arglists '([& {:keys [~@fields]}])}
         [& {:as args#}]
         (initializer# (factory# (merge (defaults#) args#))))
       record#)))
